//----------------------------------------------------------------------
//MODEULE: at45db081.h
//COMMON DESCRIPTION:
//  This module contain headers for at45db081.c
//----------------------------------------------------------------------
#ifdef AT45DB081_ENABLE
//----------------------------------------------------------------------
  void at45db081_init();
  #define at45db081_init_mac() at45db081_init()
//----------------------------------------------------------------------
  //--------------------------------------------------------------------
  // Wait for ready flash
  void at45db081_wait();
  //--------------------------------------------------------------------
  // Open flash. Must be called before
  // read and write operations
  // Read page 0 and check filesystem
  // 	Return -1, if no filesystem found,
  //	if all ok - return 0
  signed int at45db081_open();
  //--------------------------------------------------------------------
  // Close flash. Synchornize cache and flash.
  // Must be called before swich of power, if
  // data was written to flash.
  //	Return:
  //		-1, if flash not open
  //		0,  if all ok.
  signed int at45db081_close();
  //--------------------------------------------------------------------
  // Read from flash size bytes and put it to buf
  // Return:
  //	-1 - error (flash not open, for example)
  //	number of read bytes, if all ok
  signed long int  at45db081_read(char* buf, signed long int size);
  //--------------------------------------------------------------------
  // Write size bytes from buf to flash
  // Return:
  //	-1 - error (flash not open, for example)
  //	number of written bytes, if all ok
  signed long int  at45db081_write(char* buf, signed long int size);
  //--------------------------------------------------------------------
  // Format flash
  // Create structures of filesystem description
  // and write it to page 0
  void at45db081_format_mbr();
  //--------------------------------------------------------------------
  // Set file pointer to position pos.
  // Return:
  //	-1, if error;
  //	current position, if all ok;
  //--------------------------------------------------------------------
  signed long int  at45db081_seek(signed long int pos);
  //--------------------------------------------------------------------
  // Return file size, if flash opened
  //--------------------------------------------------------------------
  signed long int at45db081_fsize();
  //--------------------------------------------------------------------
  
//----------------------------------------------------------------------
#else
  #define at45db081_init_mac()
#endif
//----------------------------------------------------------------------
