//----------------------------------------------------------------------
//MODEULE: key16.c
//COMMON DESCRIPTION:
//  This module contain declaration for used matrix keyboard (16 keys)
//----------------------------------------------------------------------
// Common declarations
//----------------------------------------------------------------------
#ifdef KEY16_ENABLED
  //--------------------------------------------------------------------
  // Keyboard                             0   1   2   3 (inputs)
  unsigned char key16_recode[] PROGMEM ={'1','2','3','A', // 4 (output)
                                         '4','5','6','B', // 5 (output)
				         '7','8','9','C', // 6 (output)
					 '*','0','#','D'  // 7 (output)
					};
  //--------------------------------------------------------------------
  unsigned char key16_rowmask;
  unsigned char key16_row;
  unsigned char key16_to;
  signed int    key16_lastk;
  //
  unsigned char key16_flags;
  //--------------------------------------------------------------------
  // Shift key on
  //--------------------------------------------------------------------
  void key16_shift_on(unsigned char mshift)
    {_CLI();
     key16_flags = (key16_flags & ~(1<<(mshift+1))) | (1<<mshift);
     key16_lastk=-1;
     _SEI();
    }
  //--------------------------------------------------------------------
  // Shift key off
  //--------------------------------------------------------------------
  void key16_shift_off(unsigned char mshift)
    {_CLI();
     key16_flags &= ~((1<<mshift) | (1<<(mshift+1)));
     key16_lastk=-1;
     _SEI();
    }
  //--------------------------------------------------------------------
  // Init keyboard
  //--------------------------------------------------------------------
  void key16_init()
    {KEY16_DDR  = 0xF0;_NOP();_NOP();
     KEY16_PORT = 0xFF;_NOP();_NOP();
     key16_rowmask = 0xEF;
     key16_row=0;
     key16_lastk=-1;
     key16_to=0;
     key16_flags=0;
    }
  //--------------------------------------------------------------------
  // interrupt (period 10ms)
  //--------------------------------------------------------------------
  void _key16_timer()
    {unsigned char c=0;
     unsigned char sc,scm;
     //
     if(key16_to){key16_to--;}
     if((!key16_to)&&(key16_lastk<0))
       {// Shift row mask
        key16_rowmask = (key16_rowmask << 1) | 1; key16_row+=4;
        if(key16_rowmask==0xFF){key16_rowmask = 0xEF;key16_row=0;}
        // Get row stay
        KEY16_PORT = key16_rowmask;_NOP();_NOP();
        scm = sc = KEY16_PIN & 0x0F;
        // Find pressed key (only one key must be founded)
        while(sc & 0x01){c++; sc = sc>>1;}
	//
	if((key16_flags & (1<<KEY16_SHIFT_0)) && (c<0x04))
	  {// Shift ?
	   if( (!(key16_rowmask & (1<<(KEY16_SHIFT0_ROW+0x04)))) &&
	      (!(scm & (1<<KEY16_SHIFT0_COL))) )
	     {// yes
	      key16_flags |= (1<<KEY16_SHIFT_0P);
	     }
	   else
	     {// no
	      if(key16_flags & (1<<KEY16_SHIFT_0P))
	        {key16_lastk=PRG_RDB(key16_recode+key16_row+c) << 0x08;
		 key16_flags &= ~(1<<KEY16_SHIFT_0P);
		}
	      else
	        {key16_lastk=PRG_RDB(key16_recode+key16_row+c);}
	      //
	      key16_to=KEY16_KTIME;
	     }
	  }
	else
	  {if(c<0x04)
	     {key16_lastk=PRG_RDB(key16_recode+key16_row+c);
	      key16_to=KEY16_KTIME;
	     }
	  }
       }
    }
// Set task
settask(_key16_timer)
//----------------------------------------------------------------------
// Return -1 if no key pressed or key code
//----------------------------------------------------------------------
signed int key16_get()
  {_CLI();
   signed int c=key16_lastk;
   key16_lastk=-1;
   _SEI();
   return(c);
  }
//----------------------------------------------------------------------
#endif /*  KEY16_ENABLED */
