//MODEULE: wp1602_ni15.c
//COMMON DESCRIPTION:
//  This module contain supprt PLED modeule  WP1602 (16symbols, 2 Lines),
// connected with NI-15 bus.
//  ----------------------------------------
//  WP1602 sygnal     |         NI-15 sygnal
//  ----------------------------------------
//  1. Vss (0V)       |    1.  GND (0V)
//  2. Vdd (+5V)      |    15. Vcc (+5V)
//  3. Vbt (bright)   |    0V - +5V (variable)
//  4. RS             |    4.  M_IO
//  5. R_W            |    1.  GND (0V)
//  6. E              |    6.  _WR or adress
//                    |        decoder 
//  7. DB0            |    7.  D0
//  8. DB1            |    8.  D1
//  9. DB2            |    9.  D2
//  10.DB3            |    10. D3
//  11.DB4            |    11. D4
//  12.DB5            |    12. D5
//  13.DB6            |    13. D6
//  14.DB7            |    14. D7
//  ----------------------------------------
//----------------------------------------------------------------------
// Common declarations
//----------------------------------------------------------------------
#ifdef WP1602_NI15_ENABLED
//----------------------------------------------------------------------
unsigned char wp1602_ni15_st;   // Stay of cursor
unsigned char wp1602_ni15_dadr; // Current adr in DDRAM
//----------------------------------------------------------------------
  void wp1602_ni15_init()
    {// In init-time interrupts are disabled
     // Enable it
     _SEI();
     // Init Sequence for WP1602A
     delay_ms(20); // Dely 20ms
     // Init seq
     ni_15_wrio(WP1602_NI15_ADR, 0x30);delay_ms(5);
     ni_15_wrio(WP1602_NI15_ADR, 0x30);delay_ms(2);
     ni_15_wrio(WP1602_NI15_ADR, 0x30);delay_ms(2);
     // Font, cursor, clear etc.
     ni_15_wrio(WP1602_NI15_ADR, 0x38);delay_ms(2);
     ni_15_wrio(WP1602_NI15_ADR, 0x08);delay_ms(2);
     ni_15_wrio(WP1602_NI15_ADR, 0x01);delay_ms(2);
     ni_15_wrio(WP1602_NI15_ADR, 0x04);delay_ms(2);
     ni_15_wrio(WP1602_NI15_ADR, 0x01);delay_ms(2);
     // display on
     #ifdef WP1602_NI15_BLINK_CURSOR
       wp1602_ni15_st |= 0x0F;
       ni_15_wrio(WP1602_NI15_ADR, wp1602_ni15_st);delay_ms(2);
     #else
       #ifdef WP1602_NI15_SHOW_CURSOR
         wp1602_ni15_st = 0x0E;
         ni_15_wrio(WP1602_NI15_ADR, wp1602_ni15_st);delay_ms(2);
       #else
         wp1602_ni15_st = 0x0C;
         ni_15_wrio(WP1602_NI15_ADR, wp1602_ni15_st);delay_ms(2);
       #endif
     #endif
     // Start message
     #ifdef WP1602_NI15_SM0
       wp1602_ni15_startmes();
     #else
       #ifdef WP1602_NI15_SM1
         wp1602_ni15_startmes();
       #endif
     #endif
     // Disable ints for normal init sequence continue
     _CLI();
     // 
     wp1602_ni15_dadr=0;
    }
//----------------------------------------------------------------------
// Clear pled
void wp1602_ni15_clear()
  {wp1602_ni15_dadr=0;
   ni_15_wrio(WP1602_NI15_ADR, 0x01);delay_ms(2); // clear
  }
// show cursor
// if blink=1 cursor is blinks
void wp1602_ni15_showc(char blink)
  {if(blink)
     {wp1602_ni15_st |= 0x0F;
      ni_15_wrio(WP1602_NI15_ADR, wp1602_ni15_st);delay_ms(2);
     }
   else
     {wp1602_ni15_st |= 0x0E;
      ni_15_wrio(WP1602_NI15_ADR, wp1602_ni15_st);delay_ms(2);
     }
  }
//----------------------------------------------------------------------
// Hide cursor
void wp1602_ni15_hidec()
  {wp1602_ni15_st = 0x0C;
   ni_15_wrio(WP1602_NI15_ADR, wp1602_ni15_st);delay_ms(2);
  }
//----------------------------------------------------------------------
// Set cursor position to (x,y)
void wp1602_ni15_curat(unsigned char x, unsigned char y)
  {if( (x<WP1602_NI15_COLS) && (y<WP1602_NI15_ROWS) )
     {wp1602_ni15_dadr = ((WP1602_NI15_BPR*y) + x);
      ni_15_wrio(WP1602_NI15_ADR, 0x80 | wp1602_ni15_dadr);
      delay_ms(2);
     }
  }
//----------------------------------------------------------------------
// Write char (one byte) to memory
// Before must be setted CGRAM or DDRAM
int wp1602_ni15_outc(char ch)
  {ni_15_wrmem(WP1602_NI15_ADR, ch);
   wp1602_ni15_dadr++;
   delay_ms(2);
   return(0);
  }
//----------------------------------------------------------------------
// Write string from RAM
void wp1602_ni15_outsram(unsigned char* ram_ch)
  {ni_15_wrio(WP1602_NI15_ADR, 0x80 | wp1602_ni15_dadr);delay_ms(2);
   while(*ram_ch)
     {wp1602_ni15_outc(*(ram_ch++));}
  }
//----------------------------------------------------------------------
// Write string from FLASH PROGRAM ROM
void wp1602_ni15_outsrom(unsigned char* rom_ch)
  {ni_15_wrio(WP1602_NI15_ADR, 0x80 | wp1602_ni15_dadr);delay_ms(2);
   while(PRG_RDB(rom_ch))
     {wp1602_ni15_outc(PRG_RDB(rom_ch++));}
  }
//----------------------------------------------------------------------
// Set char from ram. Char - is image 5x8 pixels.
//   charnum - number of char (ASCII code 0x08 to 0x0F must be for  wp1602)
//   bitmap  - pointer to array, contained image of char. This array 
// must contain 8 bytes. Only 5 lower bits of these bytes is meaning.
// If pixel light - bit=1, else bit=0. Left pixel according low bit of byte
void wp1602_ni15_setchar(unsigned char charnum, unsigned char* bitmap)
  {unsigned char i;
   for(i=0;i<8;i++)
     {ni_15_wrio(WP1602_NI15_ADR, 0x40 | ((charnum<<0x03)+i));delay_ms(2);
      wp1602_ni15_outc(bitmap[i]);
     }
  }
//----------------------------------------------------------------------
// Same as function wp1602_ni15_setchar(unsigned char charnum, unsigned char* bitmap),
// but bitmaps being in the PROGRAM FLASH ROM
void wp1602_ni15_setchar_rom(unsigned char charnum, unsigned char* rombitmap)
  {unsigned char i;
   unsigned char bmp[8];
   for (i=0;i<8;i++)
     {bmp[i] = PRG_RDB(rombitmap++);
     }
   wp1602_ni15_setchar(charnum, bmp);
  }
//----------------------------------------------------------------------
// Start message
  #ifdef WP1602_NI15_SM0
    void wp1602_ni15_startmes()
      {wp1602_ni15_clear();
       wp1602_ni15_curat(0,0);
       wp1602_ni15_outsrom(PSTR(WP1602_NI15_SM0));
       #ifdef WP1602_NI15_SM1
       wp1602_ni15_curat(0,1);
       wp1602_ni15_outsrom(PSTR(WP1602_NI15_SM1));
       #endif
      }
  #else
    #ifdef WP1602_NI15_SM1
      void wp1602_ni15_startmes()
        {wp1602_ni15_clear();
	 wp1602_ni15_curat(0,1);
         wp1602_ni15_outsrom(PSTR(WP1602_NI15_SM1));
        }
    #endif
  #endif
//----------------------------------------------------------------------
#endif
//----------------------------------------------------------------------
