//----------------------------------------------------------------------
//MODEULE: wp1602_ni15.h
//COMMON DESCRIPTION:
//    This module contain headers for wp1602_ni15.c 
//----------------------------------------------------------------------
//  This module contain supprt PLED modeule  WP1602 (16symbols, 2 Lines),
// connected with NI-15 bus.
//  ----------------------------------------
//  WP1602 sygnal     |         NI-15 sygnal
//  ----------------------------------------
//  1. Vss (0V)       |    1.  GND (0V)
//  2. Vdd (+5V)      |    15. Vcc (+5V)
//  3. Vbt (bright)   |    0V - +5V (variable)
//  4. RS             |    4.  M_IO
//  5. R_W            |    1.  GND (0V)
//  6. E              |    6.  _WR or adress
//                    |        decoder 
//  7. DB0            |    7.  D0
//  8. DB1            |    8.  D1
//  9. DB2            |    9.  D2
//  10.DB3            |    10. D3
//  11.DB4            |    11. D4
//  12.DB5            |    12. D5
//  13.DB6            |    13. D6
//  14.DB7            |    14. D7
//  ----------------------------------------
//----------------------------------------------------------------------
// Common declarations
//----------------------------------------------------------------------
#ifdef WP1602_NI15_ENABLED
  void wp1602_ni15_init();
  #define wp1602_ni15_init_mac() wp1602_ni15_init()
  // Clear pled
  void wp1602_ni15_clear();
  // show cursor
  // if blink=1 cursor is blinks
  void wp1602_ni15_showc(char blink);
  // Hide cursor
  void wp1602_ni15_hidec();
  // Set cursor position to (x,y)
  void wp1602_ni15_curat(unsigned char x, unsigned char y);
  // Write char (one byte) to memory
  // Before must be setted CGRAM or DDRAM
  int wp1602_ni15_outc(char ch);
  // Write string from RAM
  void wp1602_ni15_outsram(unsigned char* ram_ch);
  // Write string from FLASH PROGRAM ROM
  void wp1602_ni15_outsrom(unsigned char* rom_ch);
  // Set char from ram. Char - is image 5x8 pixels.
  //   charnum - number of char (ASCII code 0x00 to 0x08 must be for  wp1602)
  //   bitmap  - pointer to array, contained image of char. This array 
  // must contain 8 bytes. Only 5 lower bits of these bytes is meaning.
  // If pixel light - bit=1, else bit=0. Left pixel according low bit of byte
  void wp1602_ni15_setchar(unsigned char charnum, unsigned char* bitmap);
  // Same as function wp1602_ni15_setchar(unsigned char charnum, unsigned char* bitmap),
  // but bitmaps being in the PROGRAM FLASH ROM
  void wp1602_ni15_setchar_rom(unsigned char charnum, unsigned char* rombitmap);
  // Start message
  #ifdef WP1602_NI15_SM0
    void wp1602_ni15_startmes();
  #else
    #ifdef WP1602_NI15_SM1
      void wp1602_ni15_startmes();
    #endif
  #endif
  //  ----------------------------------------
  // Common macros
  //  ----------------------------------------
  // Clear screen
  #ifndef CLR_SCR
    #define CLR_SCR() wp1602_ni15_clear()
  #endif
  // Cursor position
  #ifndef CURAT
    #define CURAT(x,y) wp1602_ni15_curat(x, y)
  #endif
  // Write symbol
  #ifndef PUTCHAR
    #define PUTCHAR(c) wp1602_ni15_outc(c)
  #endif
  // Write string (from ram)
  #ifndef PUTSTR
    #define PUTSTR(s) wp1602_ni15_outsram(s)
  #endif
  // Write string (from rom)
  #ifndef PUTPSTR
    #define PUTPSTR(s) wp1602_ni15_outsrom(s)
  #endif
  //
#else
  #define wp1602_ni15_init_mac()
#endif
//----------------------------------------------------------------------
