//----------------------------------------------------------------------
//MODEULE: mlan.c
//COMMON DESCRIPTION:
//  This module contain procedures for transmiting and recieving data
//to and from one-wire bus. 
//It contain procedure for sending reset signal also.
//----------------------------------------------------------------------
#include "mlan.h"
//----------------------------------------------------------------------
#ifdef MICROLAN_MASTER_ENABLE
// Timing
// F_INT = interrupt frequency (With it frequency call mlan_int() procedure)
// Usually, F_INT = F_CLK/N; (F_CLK - clock frecuency of chip)
#define F_INT   F_CLK/256
// All time intervals of one-wire protocol clculates from F_INT (t(sec)=1/K)
// 0.83ms (for master 0.03ms reserved)
#define ML_RESET_TIME  (unsigned char)(F_INT/1200)
// 0.5ms
#define ML_BIT_TIME    (unsigned char)(F_INT/2000)
//0.3ms
#define ML_RECBIT_TIME (unsigned char)(F_INT/3333)
//0.1ms
#define ML_SYNC_TIME   (unsigned char)(F_INT/10000)
// Pause between bits (in ML_SYNC_TIME*ML_BIT_PAUSE) (Must be ML_BIT_PAUSE >=0x01 !)
#define ML_BIT_PAUSE   0x02
// Mlan commands
#define MLAN_CMD_NONE     0x00
#define MLAN_CMD_TRANSFER 0x01
//
// MLAN_CTRL FORMAT :
// Bits 0..4: command (MLAN_CMD_XXX)
// if bit 7 is set - command is done
// Bits 6..5: current operation
// [6..5]= 0 - none, must be send reset
// [6..5]= 1 - Reset sending
// [6..5]= 2 - Data writting
// [6..5]= 3 - Data reading
// Bit 4: if sended - transfer must be terminated
unsigned char  MLAN_CTRL;     // Control byte
unsigned char  MLAN_BYTE_CNT; // Counter of bytes
unsigned char  MLAN_BIT_CNT;  // Counter of bits
unsigned char  MLAN_TIMER;    // Timer (in 0.2*T), where T is time of bit transfer
unsigned char  MLAN_RSB_TIME; // Time to reset bus
unsigned char* MLAN_BUFFER;   // Buffer fo transfer
unsigned char  MLAN_ANSWER;   // Number of bytes of answer
unsigned char  MLAN_SEND;     // Number of bytes of query
unsigned char  MLAN_LINE;     // Mask for line =~(1 << LINE_NUMBER)
unsigned char  MLAN_BYTE;     // BYTE for MLAN reading
//
unsigned char MLAN_BUF[MLAN_BUF_LEN]; // Reserved for mlan buffer
//----------------------------------------------------------------------
// Initialize protocol MLAN
void mlan_init()
  {// Common
   MLAN_CTRL=MLAN_CMD_NONE;
   MLAN_LINE=0xFF;
   #ifdef MICROLAN_MASTER_EXTPORTS_ENABLED
   extio_wroport(MLANPORT,0xFF);
   #else
   output(MLANPORT,input(MLANPORT) | (1<<MLAN_OUTBIT));_NOP();_NOP();
   output(MLAN_PORTDDR ,input(MLAN_PORTDDR) | (1<<MLAN_OUTBIT));_NOP();_NOP();
   output(MLAN_PINDDR ,input(MLAN_PINDDR) & ~(1<<MLAN_INBIT));_NOP();_NOP();
   #endif
  }
//----------------------------------------------------------------------
// Send command and data to mlan bus and recieved answer from device.
// Before call mlan_set_trs in buffer must be stored data for send to bus.
// After transfer in buffer stored recieved data.
// answer - number of bytes, which must be recieved from bus after sending query 
// (0 if no answer excepted)
// send - number of bytes, which must be send to bus from buffer (never is 0 !)
// Return true is transfer beginning and false if transfer can't be setted
// (previous transfer not terminated, for example).
unsigned char mlan_set_trs(unsigned char* buffer,unsigned char line,unsigned char send,unsigned char answer)
  {if(!is_mlan_done()){return(0);/* return false, do nothing */}
   else
     {MLAN_BUFFER = buffer; // Buffer adress
      MLAN_ANSWER = answer; // Number of recieved bytes
      MLAN_SEND   = send;   // Number of transmited bytes
      MLAN_LINE   = ~(1<<line);// Mask for line
      MLAN_CTRL   = MLAN_CMD_TRANSFER; // Send Command
      return(1);
     }
  }
//----------------------------------------------------------------------
// MLAN Interrupt handler
void mlan_int()
  {//
   if(MLAN_TIMER){MLAN_TIMER--;}
   else if(MLAN_CTRL & 0x10)
     {// Command is done
      MLAN_CTRL=(MLAN_CTRL & 0x0F) | 0x80;
      #ifdef MICROLAN_MASTER_EXTPORTS_ENABLED
      extio_wroport(MLANPORT,0xFF);
      #else
      output(MLANPORT,input(MLANPORT) | (1<<MLAN_OUTBIT));_NOP();_NOP();
      output(MLAN_PORTDDR ,input(MLAN_PORTDDR) | (1<<MLAN_OUTBIT));_NOP();_NOP();
      output(MLAN_PINDDR ,input(MLAN_PINDDR) & ~(1<<MLAN_INBIT));_NOP();_NOP();
      #endif
     }
   // Reset line   
   if(MLAN_TIMER<=MLAN_RSB_TIME)
     {//
      #ifdef MICROLAN_MASTER_EXTPORTS_ENABLED
      extio_wroport(MLANPORT,0xFF);
      #else
      output(MLANPORT,input(MLANPORT) | (1<<MLAN_OUTBIT));_NOP();_NOP();
      output(MLAN_PORTDDR ,input(MLAN_PORTDDR) | (1<<MLAN_OUTBIT));_NOP();_NOP();
      output(MLAN_PINDDR ,input(MLAN_PINDDR) & ~(1<<MLAN_INBIT));_NOP();_NOP();
      #endif
     }
   // Writting ?
   if(!(MLAN_CTRL & 0x80)) // Test bit 7 (is command done ?)
     {if((MLAN_CTRL & 0x0F)==MLAN_CMD_TRANSFER) // Is command MLAN_CMD_TRANSFER ?
        {// Reset
	 if((MLAN_CTRL & 0x60)==0x00)
	   {// Sending reset
	    #ifdef MICROLAN_MASTER_EXTPORTS_ENABLED
	    extio_wroport(MLANPORT,MLAN_LINE);
	    #else
	    output(MLAN_PORTDDR ,input(MLAN_PORTDDR) | (1<<MLAN_OUTBIT));_NOP();_NOP();
	    output(MLANPORT,input(MLANPORT) & ~(1<<MLAN_OUTBIT));_NOP();_NOP();
	    #endif
	    MLAN_TIMER    = (ML_RESET_TIME+ML_SYNC_TIME);
	    MLAN_RSB_TIME = ML_SYNC_TIME;
	    MLAN_CTRL=(MLAN_CTRL & 0x1F) | 0x20;
	   }	   
	 else if((MLAN_CTRL & 0x60)==0x20)
	   {// Reset sended, Send data.	    
	    if(!MLAN_TIMER)
	      {MLAN_CTRL=(MLAN_CTRL & 0x1F) | 0x40;
	       MLAN_BYTE_CNT = MLAN_SEND;
               MLAN_BIT_CNT =0x08;
	      }
	   }
	 // Send
	 if((MLAN_CTRL & 0x60)==0x40)
	   {// Sending data
	    if(!MLAN_TIMER)
	      {if(MLAN_BIT_CNT)
	        {// Sending bit
		 #ifdef MICROLAN_MASTER_EXTPORTS_ENABLED
		 extio_wroport(MLANPORT,MLAN_LINE);
		 #else
		 output(MLAN_PORTDDR ,input(MLAN_PORTDDR) | (1<<MLAN_OUTBIT));_NOP();_NOP();
	         output(MLANPORT,input(MLANPORT) & ~(1<<MLAN_OUTBIT));_NOP();_NOP();
	         #endif
		 // 0 or 1 ???
		 if(MLAN_BUFFER[MLAN_SEND - MLAN_BYTE_CNT] & (1 << (8-MLAN_BIT_CNT)))
		   {MLAN_RSB_TIME = ML_BIT_TIME+((ML_BIT_PAUSE-1)*ML_SYNC_TIME);}
		 else
		   {MLAN_RSB_TIME = ML_BIT_PAUSE*ML_SYNC_TIME;}
		 MLAN_TIMER = (ML_BIT_TIME+(ML_BIT_PAUSE*ML_SYNC_TIME));
		 // Next Byte ?
		 MLAN_BIT_CNT--;
		 if(!MLAN_BIT_CNT)
		   {MLAN_BYTE_CNT--;
		    // All bytes sended ?
		    if(!MLAN_BYTE_CNT)
		      {if(!MLAN_ANSWER)
		         {MLAN_CTRL=MLAN_CTRL | 0x10;}
		       else
		         {MLAN_CTRL=(MLAN_CTRL & 0x0F) | 0x60;
			  MLAN_BYTE_CNT = MLAN_ANSWER;
                          MLAN_BIT_CNT =0x09; // one falshbit
			 }
		      }
		    else
		      {// Next byte
		       MLAN_BIT_CNT =0x08;
		      }
		   }
	        }
	      }
	   }
	 //
	}
     }   
   // Reading byte ?
   if((MLAN_CTRL & 0xF0)==0x60)
     {// Reading data      
      if(!MLAN_TIMER)
        {//
	 MLAN_BIT_CNT--;
	 if(!MLAN_BIT_CNT)
	   {MLAN_BIT_CNT =0x08;
	    MLAN_BUFFER[MLAN_ANSWER-MLAN_BYTE_CNT]=MLAN_BYTE;	   
	    MLAN_BYTE_CNT--;
	    // All bytes recieved ?
            if(!MLAN_BYTE_CNT)
	      {MLAN_CTRL=MLAN_CTRL | 0x10;
	      }
	   }	 
	 // Send sync ??
	 if(!(MLAN_CTRL & 0x10))
	   {//
	    #ifdef MICROLAN_MASTER_EXTPORTS_ENABLED
	    extio_wroport(MLANPORT,MLAN_LINE);
	    #else
	    output(MLAN_PORTDDR ,input(MLAN_PORTDDR) | (1<<MLAN_OUTBIT));_NOP();_NOP();
	    output(MLANPORT,input(MLANPORT) & ~(1<<MLAN_OUTBIT));_NOP();_NOP();
	    #endif
	    MLAN_RSB_TIME = (ML_BIT_TIME+((ML_BIT_PAUSE-1)*ML_SYNC_TIME));
	    MLAN_TIMER = (ML_BIT_TIME+(ML_BIT_PAUSE*ML_SYNC_TIME));	    
	   }
	}
        else if(MLAN_TIMER== (ML_BIT_TIME + (ML_BIT_PAUSE*ML_SYNC_TIME) - ML_RECBIT_TIME ) )
        {MLAN_BYTE=MLAN_BYTE>>1;
	 #ifdef MICROLAN_MASTER_EXTPORTS_ENABLED
	 if(extio_rdiport(MLANPIN) & (~MLAN_LINE)){MLAN_BYTE=MLAN_BYTE | 0x80;}
	 else {MLAN_BYTE=MLAN_BYTE & 0x7F;}
	 #else
	 if(input(MLANPIN) & (1<<MLAN_INBIT)){MLAN_BYTE=MLAN_BYTE | 0x80;}
	 else {MLAN_BYTE=MLAN_BYTE & 0x7F;}
	 #endif
	}           
     }
  } 
//----------------------------------------------------------------------
// Send data to bus (automatic generate CRC) 
// Packet: adr,cmd,data,crc
// adr - device adress 
// cmd - command
// len - length of data
// line - number of line
// data - adress (NULL if command only sending) of data block
unsigned char mlan_transfer(unsigned char adr,unsigned char cmd,unsigned char len,
                            unsigned char line,unsigned char* data, unsigned char answ)
  {unsigned char crc=0xAA;
   unsigned char i,k;
   MLAN_BUF[0]=adr;crc=crc+adr;
   MLAN_BUF[1]=cmd;crc=crc+cmd;
   for(i=0;i<len;i++){k=data[i];crc=crc+k;MLAN_BUF[i+2]=k;}
   MLAN_BUF[len+2]=crc;
   //
   if(answ){answ++;}
   return(mlan_set_trs(MLAN_BUF,line,len+3,answ));
  }
//----------------------------------------------------------------------
// Calculate answercrc (true/false)
unsigned char mlan_answer_crcansw()
  {unsigned char crc=0xAA;
   unsigned char i,rt;
   for(i=0;i<(MLAN_ANSWER-1);i++){crc=crc+MLAN_BUF[i];}
   //
   if(MLAN_BUF[MLAN_ANSWER-1]==crc){rt=0x01;}else{rt=0x00;}
   //
   return(rt);
  }
//----------------------------------------------------------------------
// Returns true if command is not executing and clearing MLAN_CTRL
unsigned char is_mlan_done()
  {if((MLAN_CTRL & 0x80) || ((MLAN_CTRL & 0x0F)==0x00))
     {MLAN_CTRL=MLAN_CMD_NONE;return(0x01);}
   else
     {return(0x00);}
  }  
//----------------------------------------------------------------------
#endif
